@@ -1,6 +1,6 @@
-/* 
+/*
  * gnc-plugin-page.h -- A page, which can be added to the
- *	GnuCash main window.
+ *  GnuCash main window.
  *
  * Copyright (C) 2003 Jan Arne Petersen <jpetersen@uni-bonn.de>
  * Copyright (C) 2003,2005 David Hampton <hampton@employees.org>
@@ -41,7 +41,8 @@
 
 G_BEGIN_DECLS
 
-#define KEY_SUMMARYBAR_POSITION "summarybar_position"
+#define GNC_PREF_SUMMARYBAR_POSITION_TOP    "summarybar-position-top"
+#define GNC_PREF_SUMMARYBAR_POSITION_BOTTOM "summarybar-position-bottom"
 
 /* type macros */
 #define GNC_TYPE_PLUGIN_PAGE            (gnc_plugin_page_get_type ())
@@ -54,134 +55,152 @@ G_BEGIN_DECLS
 /* typedefs & structures */
 
 /** The instance data structure for a content plugin. */
-typedef struct GncPluginPage {
-	GObject gobject;		/**< The parent object data. */
+typedef struct GncPluginPage
+{
+    GObject gobject;            /**< The parent object data. */
 
-	GtkWidget *window;		/**< The window that contains the
-					 *   display widget for this plugin.
-					 *   This field is private to the
-					 *   gnucash window management
-					 *   code.  */
-	GtkWidget *notebook_page;	/**< The display widget for this
-					 *   plugin.  This field is private to
-					 *   the gnucash window management
-					 *   code.  */
-	GtkWidget *summarybar;		/**< The summary bar widget (if any)
-					 *   that is associated with this
-					 *   plugin.  This field is private to
-					 *   the gnucash window management
-					 *   code.  */
+    GtkWidget *window;          /**< The window that contains the
+                                *   display widget for this plugin.
+                                *   This field is private to the
+                                *   gnucash window management
+                                *   code.  */
+    GtkWidget *notebook_page;   /**< The display widget for this
+                                *   plugin.  This field is private to
+                                *   the gnucash window management
+                                *   code.  */
+    GtkWidget *summarybar;      /**< The summary bar widget (if any)
+                                *   that is associated with this
+                                *   plugin.  This field is private to
+                                *   the gnucash window management
+                                *   code.  */
 } GncPluginPage;
 
 
 /** The class data structure for a content plugin. */
-typedef struct {
-	GObjectClass gobject;
+typedef struct
+{
+    GObjectClass gobject;
 
-	/** The relative name of the icon that should be shown on the
-	 *  tab for this page. */
-	const gchar *tab_icon;
-	/** The textual name of this plugin. */
-	const gchar *plugin_name;
+    /** The relative name of the icon that should be shown on the
+     *  tab for this page. */
+    const gchar *tab_icon;
+    /** The textual name of this plugin. */
+    const gchar *plugin_name;
 
-	/* Signals */
-	void (* inserted) (GncPluginPage *plugin_page);
-	void (* removed) (GncPluginPage *plugin_page);
-	void (* selected) (GncPluginPage *plugin_page);
-	void (* unselected) (GncPluginPage *plugin_page);
+    /* Signals */
+    void (* inserted) (GncPluginPage *plugin_page);
+    void (* removed) (GncPluginPage *plugin_page);
+    void (* selected) (GncPluginPage *plugin_page);
+    void (* unselected) (GncPluginPage *plugin_page);
 
-	/* Virtual Table */
+    /* Virtual Table */
 
-	/** Function called to create the display widget for a
-	 *  particular type of plugin.  The returned widget should
-	 *  encompass all information that goes with this page,
-	 *  including scroll bars, a summary bar, etc.
-	 *
-	 *  @param plugin_page A pointer to the plugin for which a
-	 *  display widget should be created.
-	 *
-	 *  @return A displayable gtk widget. */
-	GtkWidget *(* create_widget) (GncPluginPage *plugin_page);
-	/** Function called to destroy the display widget for a
-	 *  particular type of plugin.
-	 *
-	 *  @param plugin_page A pointer to the plugin whose display
-	 *  widget should be destroyed. */
-	void (* destroy_widget) (GncPluginPage *plugin_page);
+    /** Function called to create the display widget for a
+     *  particular type of plugin.  The returned widget should
+     *  encompass all information that goes with this page,
+     *  including scroll bars, a summary bar, etc.
+     *
+     *  @param plugin_page A pointer to the plugin for which a
+     *  display widget should be created.
+     *
+     *  @return A displayable gtk widget. */
+    GtkWidget *(* create_widget) (GncPluginPage *plugin_page);
 
-	/** Save enough information about this page so that it can be
-	 *  recreated next time the user starts gnucash.
-	 *  
-	 *  @param page The page to save.
-	 *
-	 *  @param key_file A pointer to the GKeyFile data structure where the
-	 *  page information should be written.
-	 *
-	 *  @param group_name The group name to use when writing data.
-	 *  The name is specific to this page instance. */
-	void (* save_page) (GncPluginPage *page, GKeyFile *file, 
-                            const gchar *group);
+    /** Function called to destroy the display widget for a
+     *  particular type of plugin.
+     *
+     *  @param plugin_page A pointer to the plugin whose display
+     *  widget should be destroyed. */
+    void (* destroy_widget) (GncPluginPage *plugin_page);
 
-	/** Create a new page based on the information saved during a
-	 *  previous instantiation of gnucash.  This function may or
-	 *  may not install the new page in the window as it sees fit.
-	 *  Generally the function will install the page int the
-	 *  window in order to manipulate the menu items that are
-	 *  created at install time.
-	 *
-	 *  @param window The window where this new page will be
-	 *  installed.
-	 *
-	 *  @param key_file A pointer to the GKeyFile data structure where the
-	 *  page information should be retrieved.
-	 *
-	 *  @param group_name The group name to use when retrieving
-	 *  data.  The name is specific to this page instance.
-	 *
-	 *  @return A pointer to the new page. */
-        GncPluginPage * (* recreate_page) (GtkWidget *window, GKeyFile *file, 
-                                           const gchar *group);
+    /** Save enough information about this page so that it can be
+     *  recreated next time the user starts gnucash.
+     *
+     *  @param page The page to save.
+     *
+     *  @param key_file A pointer to the GKeyFile data structure where the
+     *  page information should be written.
+     *
+     *  @param group_name The group name to use when writing data.
+     *  The name is specific to this page instance. */
+    void (* save_page) (GncPluginPage *page, GKeyFile *file,
+                        const gchar *group);
 
-	/** Perform plugin specific actions when a page is added to a
-	 *  window (or has been removed from one window and added to a
-	 *  new window).  This function is called after the page is
-	 *  installed in the window, just before the window's
-	 *  PAGE_ADDED signal is generated.
-	 *  
-	 *  @param page The page that was added to a window.
-	 *
-	 *  @param window The window where the page was added. */
-	void (* window_changed) (GncPluginPage *plugin_page, GtkWidget *window);
+    /** Create a new page based on the information saved during a
+     *  previous instantiation of gnucash.  This function may or
+     *  may not install the new page in the window as it sees fit.
+     *  Generally the function will install the page int the
+     *  window in order to manipulate the menu items that are
+     *  created at install time.
+     *
+     *  @param window The window where this new page will be
+     *  installed.
+     *
+     *  @param key_file A pointer to the GKeyFile data structure where the
+     *  page information should be retrieved.
+     *
+     *  @param group_name The group name to use when retrieving
+     *  data.  The name is specific to this page instance.
+     *
+     *  @return A pointer to the new page. */
+    GncPluginPage * (* recreate_page) (GtkWidget *window, GKeyFile *file,
+                                       const gchar *group);
 
-	/** This function vector allows page specific actions to occur
-	 *  when the page name is changed.
-	 *  
-	 *  @param page The page to update.
-	 *  
-	 *  @param name The new name for this page. */
-	void (* page_name_changed) (GncPluginPage *plugin_page, 
-                                    const gchar *name);
+    /** Perform plugin specific actions when a page is added to a
+     *  window (or has been removed from one window and added to a
+     *  new window).  This function is called after the page is
+     *  installed in the window, just before the window's
+     *  PAGE_ADDED signal is generated.
+     *
+     *  @param page The page that was added to a window.
+     *
+     *  @param window The window where the page was added. */
+    void (* window_changed) (GncPluginPage *plugin_page, GtkWidget *window);
 
-	/** This function vector allows page specific actions to
-	 *  override the generic code for setting the sensitivity of
-	 *  items in the Edit menu.
-	 *  
-	 *  @param page The front page in a main window..
-	 *  
-	 *  @param hide Whether the widgets should be shown or
-	 *  hidden. */
-	void (* update_edit_menu_actions) (GncPluginPage *plugin_page, gboolean hide);
+    /** Perform plugin specific actions to set the focus.
+     *
+     *  @param page The page that was added to a window.
+     *
+     *  @param on_current_pgae Whether this page is the currentone. */
+    void (* focus_page) (GncPluginPage *plugin_page, gboolean on_current_page);
 
-	/** This function vector is called to finish any outstanding
-	 *  activities.  It will be called for such things as closing a
-	 *  page, saving the data file, etc.
-	 *  
-	 *  @param page The page in a main window.
-	 *
-	 *  @return FALSE if the page could not or would not comply,
-	 *  which should cancel the pending operation.  TRUE
-	 *  otherwise */
-	gboolean (* finish_pending) (GncPluginPage *plugin_page);
+    /** This function performs specific actions to set the focus on a specific
+     *  widget.
+     *
+     *  @param page The page that was added to a window.
+     *
+     *  @return FALSE to remove idle */
+    gboolean (* focus_page_function) (GncPluginPage *plugin_page);
+
+    /** This function vector allows page specific actions to occur
+     *  when the page name is changed.
+     *
+     *  @param page The page to update.
+     *
+     *  @param name The new name for this page. */
+    void (* page_name_changed) (GncPluginPage *plugin_page,
+                                const gchar *name);
+
+    /** This function vector allows page specific actions to
+     *  override the generic code for setting the sensitivity of
+     *  items in the Edit menu.
+     *
+     *  @param page The front page in a main window..
+     *
+     *  @param hide Whether the widgets should be shown or
+     *  hidden. */
+    void (* update_edit_menu_actions) (GncPluginPage *plugin_page, gboolean hide);
+
+    /** This function vector is called to finish any outstanding
+     *  activities.  It will be called for such things as closing a
+     *  page, saving the data file, etc.
+     *
+     *  @param page The page in a main window.
+     *
+     *  @return FALSE if the page could not or would not comply,
+     *  which should cancel the pending operation.  TRUE
+     *  otherwise */
+    gboolean (* finish_pending) (GncPluginPage *plugin_page);
 } GncPluginPageClass;
 
 
@@ -201,7 +220,8 @@ GType gnc_plugin_page_get_type (void);
  *  @param plugin_page A pointer to the plugin for which a display
  *  widget should be created.
  *
- *  @return A displayable gtk widget. */
+ *  @return A displayable gtk widget.
+ */
 GtkWidget *gnc_plugin_page_create_widget (GncPluginPage *plugin_page);
 
 
@@ -210,7 +230,8 @@ GtkWidget *gnc_plugin_page_create_widget (GncPluginPage *plugin_page);
  *  code when a page is closed.
  *
  *  @param plugin_page A pointer to the plugin whose display widget
- *  should be destroyed. */
+ *  should be destroyed.
+ */
 void gnc_plugin_page_destroy_widget (GncPluginPage *plugin_page);
 
 
@@ -233,10 +254,11 @@ void gnc_plugin_page_show_summarybar (GncPluginPage *page, gboolean visible);
  *  @param key_file A pointer to the GKeyFile data structure where the
  *  page information should be written.
  *
- *  @param group_name The group name to use when saving data. */
+ *  @param group_name The group name to use when saving data.
+ */
 void gnc_plugin_page_save_page (GncPluginPage *page,
-				GKeyFile *key_file,
-				const gchar *group_name);
+                                GKeyFile *key_file,
+                                const gchar *group_name);
 
 
 /** This function looks up a specific plugin type by name, and then
@@ -250,11 +272,12 @@ void gnc_plugin_page_save_page (GncPluginPage *page,
  *  @param key_file A pointer to the GKeyFile data structure where the
  *  page information should be read.
  *
- *  @param group_name The group name to use when restoring data. */
+ *  @param group_name The group name to use when restoring data.
+ */
 GncPluginPage *gnc_plugin_page_recreate_page (GtkWidget *window,
-					      const gchar *page_type,
-					      GKeyFile *key_file,
-					      const gchar *group_name);
+                                              const gchar *page_type,
+                                              GKeyFile *key_file,
+                                              const gchar *group_name);
 
 
 /** Add the actions for a content page to the specified window.
@@ -263,8 +286,9 @@ GncPluginPage *gnc_plugin_page_recreate_page (GtkWidget *window,
  *  added to the user interface.
  *
  *  @param merge A pointer to the UI manager data structure for a
- *  window. */
-void gnc_plugin_page_merge_actions (GncPluginPage *plugin_page, 
+ *  window.
+ */
+void gnc_plugin_page_merge_actions (GncPluginPage *plugin_page,
                                     GtkUIManager *merge);
 
 
@@ -274,8 +298,9 @@ void gnc_plugin_page_merge_actions (GncPluginPage *plugin_page,
  *  removed from the user interface.
  *
  *  @param merge A pointer to the UI manager data structure for a
- *  window. */
-void gnc_plugin_page_unmerge_actions (GncPluginPage *plugin_page, 
+ *  window.
+ */
+void gnc_plugin_page_unmerge_actions (GncPluginPage *plugin_page,
                                       GtkUIManager *merge);
 
 
@@ -285,7 +310,8 @@ void gnc_plugin_page_unmerge_actions (GncPluginPage *plugin_page,
  *  should be retrieved.
  *
  *  @return The name of this plugin.  This string is owned by the
- *  plugin. */
+ *  plugin.
+ */
 const gchar *gnc_plugin_page_get_plugin_name (GncPluginPage *plugin_page);
 
 
@@ -353,7 +379,7 @@ void gnc_plugin_page_set_page_name (GncPluginPage *page, const char *name);
 
 
 /** Retrieve the long name of this page.  This is the string used in
- *  the tooltip that is attached to the pate name in the notebook
+ *  the tooltip that is attached to the page name in the notebook
  *  tab.
  *
  *  @param page The page whose name should be retrieved.
@@ -365,7 +391,7 @@ const gchar *gnc_plugin_page_get_page_long_name (GncPluginPage *page);
 
 
 /** Set the long name of this page.  This is the string used in the
- *  tooltip that is attached to the pate name in the notebook tab.
+ *  tooltip that is attached to the page name in the notebook tab.
  *
  *  @param page The page whose name should be set.
  *
@@ -374,6 +400,46 @@ const gchar *gnc_plugin_page_get_page_long_name (GncPluginPage *page);
 void gnc_plugin_page_set_page_long_name (GncPluginPage *page, const char *name);
 
 
+/** Retrieve the color of this page. This is the color string used
+ *  in the notebook tab.
+ *
+ *  @param page The page whose name should be retrieved.
+ *
+ *  @return The color for this page.  This string is owned by the page and
+ *  should not be freed by the caller.
+ */
+const gchar *gnc_plugin_page_get_page_color (GncPluginPage *page);
+
+
+/** Set the color of this page. This is the color string used
+ *  in the notebook tab.
+ *
+ *  @param page The page whose name should be retrieved.
+ *
+ *  @param color The color for this page.  This string is owned by the page and
+ *  should not be freed by the caller.
+ */
+void gnc_plugin_page_set_page_color (GncPluginPage *page, const char *color);
+
+
+/** Set up the page_changed callback for when the current page is changed.
+ *  This will store a pointer to the page focus function passed as a parameter
+ *  so that it can be used in setting up the g_idle_add
+ *
+ *  @param page The page the callback is setup for.
+ *
+ *  @param user_data The page focus function
+ */
+void gnc_plugin_page_inserted_cb (GncPluginPage *page, gpointer user_data);
+
+
+/** Disconnect the page_changed_id signal callback.
+ *
+ *  @param page The page whose name should be retrieved.
+ */
+void gnc_plugin_page_disconnect_page_changed (GncPluginPage *page);
+
+
 /** Retrieve the Uniform Resource Identifier for this page.
  *
  *  @param page The page whose URI should be retrieved.
@@ -409,8 +475,8 @@ const gchar *gnc_plugin_page_get_statusbar_text (GncPluginPage *page);
  *
  *  @param name The new statusbar text for the page.
  */
-void gnc_plugin_page_set_statusbar_text (GncPluginPage *page, 
-                                         const char *name);
+void gnc_plugin_page_set_statusbar_text (GncPluginPage *page,
+        const char *name);
 
 
 /** Retrieve the "use new window" setting associated with this page.
@@ -431,8 +497,8 @@ gboolean gnc_plugin_page_get_use_new_window (GncPluginPage *page);
  *
  *  @param use_new The new value for this setting.
  */
-void gnc_plugin_page_set_use_new_window (GncPluginPage *page, 
-                                         gboolean use_new);
+void gnc_plugin_page_set_use_new_window (GncPluginPage *page,
+        gboolean use_new);
 
 
 /** Retrieve the name of the XML UI file associated with this page.
@@ -455,8 +521,8 @@ const char *gnc_plugin_page_get_ui_description (GncPluginPage *page);
  *
  *  @param ui_filename The filename (no path) of the alternate UI.
  */
-void gnc_plugin_page_set_ui_description (GncPluginPage *page, 
-                                         const char *ui_filename);
+void gnc_plugin_page_set_ui_description (GncPluginPage *page,
+        const char *ui_filename);
 
 
 /** Retrieve the GtkUIManager object associated with this page.
@@ -472,7 +538,8 @@ GtkUIManager *gnc_plugin_page_get_ui_merge (GncPluginPage *page);
  *  @param page The page whose menu/toolbar action group should be
  *  retrieved.
  *
- *  @return A pointer to the GtkActionGroup object for this page. */
+ *  @return A pointer to the GtkActionGroup object for this page.
+ */
 GtkActionGroup *gnc_plugin_page_get_action_group (GncPluginPage *page);
 
 
@@ -486,8 +553,9 @@ GtkActionGroup *gnc_plugin_page_get_action_group (GncPluginPage *page);
  *  be consistent across all pages of the same type.
  *
  *  @return A pointer to the newly created GtkActionGroup object for
- *  this page. */
-GtkActionGroup * gnc_plugin_page_create_action_group (GncPluginPage *page, 
+ *  this page.
+ */
+GtkActionGroup * gnc_plugin_page_create_action_group (GncPluginPage *page,
                                                       const gchar *group_name);
 
 /** Retrieve a GtkAction object associated with this page.
@@ -497,7 +565,8 @@ GtkActionGroup * gnc_plugin_page_create_action_group (GncPluginPage *page,
  *
  *  @param name The name of the GtkAction to find.
  *
- *  @return A pointer to the retuested GtkAction object or NULL. */
+ *  @return A pointer to the retuested GtkAction object or NULL.
+ */
 GtkAction *gnc_plugin_page_get_action (GncPluginPage *page, const gchar *name);
 
 /* Signals */
@@ -507,11 +576,12 @@ void gnc_plugin_page_selected (GncPluginPage *plugin_page);
 void gnc_plugin_page_unselected (GncPluginPage *plugin_page);
 
 /** Tell a page to finish any outstanding activities.
- *  
- *  @param page A page.
+ *
+ *  @param plugin_page A page.
  *
  *  @return FALSE if the page could not or would not comply, which
- *  should cancel the pending operation.  TRUE otherwise */
+ *  should cancel the pending operation.  TRUE otherwise
+ */
 gboolean gnc_plugin_page_finish_pending (GncPluginPage *plugin_page);
 
 G_END_DECLS
