@@ -294,7 +294,7 @@ developing over time"))
 
      (else
       (let* ((commodity-list (gnc:accounts-get-commodities
-                              (gnc:accounts-and-all-descendants accounts)
+                              (gnc-accounts-and-all-descendants accounts)
                               report-currency))
              (exchange-fn (gnc:case-exchange-time-fn
                            price-source report-currency
@@ -394,7 +394,7 @@ developing over time"))
                       #:nosplit->elt (gnc:make-gnc-monetary comm 0)))))
            ;; all selected accounts (of report-specific type), *and*
            ;; their descendants (of any type) need to be scanned.
-           (gnc:accounts-and-all-descendants accounts)))
+           (gnc-accounts-and-all-descendants accounts)))
 
         ;; Creates the <balance-list> to be used in the function
         ;; below.
@@ -526,10 +526,8 @@ Please deselect the accounts with negative balances."))
                                  (list-head dates-list (1- (length dates-list)))
                                  dates-list))
                  (date-string-list (map qof-print-date dates-list))
-                 (list-of-rows (apply zip (map cadr all-data)))
-
-                 ;; total amounts
-                 (row-totals (map (cut fold + 0 <>) list-of-rows)))
+                 (list-of-rows #f)
+                 (row-totals #f))
 
             ;; Set chart title, subtitle etc.
             (gnc:html-chart-set-type!
@@ -578,6 +576,8 @@ Please deselect the accounts with negative balances."))
                       (gnc:report-anchor-text
                        (gnc:make-report reportguid options))))))
 
+            (set! list-of-rows (apply zip (map cadr all-data)))
+            (set! row-totals (map (cut fold + 0 <>) list-of-rows))
             (gnc:report-percent-done 92)
 
             (for-each
